/*
 * $Id: menu.c,v 1.5 2004/12/30 09:07:47 ali Exp $
 * Copyright (C) 2007 J. Ali Harlow
 * Min Hack may be freely redistributed.  See license for details. 
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "minhack.h"

void minhack_free_menu(int window)
{
    struct minhack_menu *menu=windows[window].menu;
    struct minhack_mitem *item,*next;
    if (menu)
    {
	if (menu->menu)
	{
#if 0
	    delwin(menu_win(menu->menu));
	    delwin(menu_sub(menu->menu));
#endif
	    free_menu(menu->menu);
	}
	for(item=menu->mitems;item;item=next)
	{
	    next=item->next;
	    free_item(item->item);
	    free(item->str);
	    free(item);
	}
	free(menu->items);
    }
    windows[window].menu=NULL;
}

void minhack_start_menu(int window)
{
    minhack_free_menu(window);
    windows[window].menu=
      (struct minhack_menu *)calloc(1,sizeof(struct minhack_menu));
}

void minhack_add_menu(int window, int glyph, int identifier, char accelerator,
  char groupacc, int attr, const char *str, nhproxy_bool_t preselected)
{
    struct minhack_mitem *item;
    item=(struct minhack_mitem *)malloc(sizeof(*item));
    item->next=windows[window].menu->mitems;
    item->identifier=identifier;
    if (accelerator)
	item->accelerator=accelerator;
    else if (!item->next)
	item->accelerator='a';
    else if (!identifier)
	item->accelerator=item->next->accelerator;
    else if (item->next->accelerator=='z')
	item->accelerator='A';
    else
	item->accelerator=item->next->accelerator+1;
    item->groupacc=groupacc;
    if (identifier)
    {
	item->str=malloc(strlen(str)+5);
	sprintf(item->str,"%c - %s",item->accelerator,str);
    }
    else if (*str)
	item->str=strdup(str);
    else
	item->str=strdup(" ");
    item->selected=preselected;
    item->item=new_item(item->str,item->str);
    if (!identifier)
	item_opts_off(item->item,O_SELECTABLE);
    windows[window].menu->mitems=item;
}

static struct minhack_mitem *reverse(struct minhack_mitem *curr)
{
    struct minhack_mitem *next,*head=0;
    while (curr)
    {
	next=curr->next;
	curr->next=head;
	head=curr;
	curr=next;
    }
    return head;
}

void minhack_end_menu(int window, const char *prompt)
{
    int i,rows,cols;
    struct minhack_mitem *item;
    WINDOW *win;
    windows[window].menu->mitems=reverse(windows[window].menu->mitems);
    if (*prompt)
    {
	minhack_add_menu(window,-1,0,0,0,0,"",FALSE);
	minhack_add_menu(window,-1,0,0,0,0,prompt,FALSE);
    }
    for(i=0,item=windows[window].menu->mitems;item;i++,item=item->next)
	;
    windows[window].menu->items=malloc((i+1)*sizeof(ITEM *));
    for(i=0,item=windows[window].menu->mitems;item;i++,item=item->next)
	windows[window].menu->items[i]=item->item;
    windows[window].menu->items[i]=NULL;
    windows[window].menu->menu=new_menu(windows[window].menu->items);
    scale_menu(windows[window].menu->menu,&rows,&cols);
    {
	char buf[100];
	sprintf(buf,"(%dx%d)",rows,cols);
	minhack_putstr(message_win,0,buf);
    }
#if 0
    win=newwin(rows,cols,0,0);
    set_menu_win(windows[window].menu->menu,win);
#endif
    win=newwin(rows,cols,0,0);
    set_menu_sub(windows[window].menu->menu,win);
    set_menu_win(windows[window].menu->menu,windows[map_win].win);
}

int minhack_select_menu(int window, int how, struct nhproxy_mi **selected)
{
    int i,j,ch;
    struct minhack_mitem *item;
    if (how==MINHACK_PICK_ANY)
	menu_opts_off(windows[window].menu->menu,O_ONEVALUE);
    menu_opts_off(windows[window].menu->menu,O_IGNORECASE);
    menu_opts_off(windows[window].menu->menu,O_SHOWDESC);
    i=post_menu(windows[window].menu->menu);
    for(;;)
    {
	wrefresh(menu_win(windows[window].menu->menu));
#if 1
	wrefresh(menu_sub(windows[window].menu->menu));
#endif
	ch=getch();
	if (ch=='\033')
	{
	    i=how==MINHACK_PICK_NONE?0:-1;
	    break;
	}
	else if (ch>=' ' && ch<='~')
	    if (menu_driver(windows[window].menu->menu,ch)==E_OK)
	    {
		for(i=0,item=windows[window].menu->mitems;item;item=item->next)
		    if (item_value(item->item))
			i++;
		*selected=(struct nhproxy_mi *)calloc(i,
		  sizeof(struct nhproxy_mi));
		for(i=0,item=windows[window].menu->mitems;item;item=item->next)
		    if (item_value(item->item))
		    {
			(*selected)[i].item=item->identifier;
			(*selected)[i].count=-1;	/* FIXME */
		    }
		break;
	    }
    }
    unpost_menu(windows[window].menu->menu);
    redrawwin(menu_win(windows[window].menu->menu));
#if 0
    for(j=0;j<no_windows;j++)
	if (windows[j].win)
	    redrawwin(windows[j].win);
#endif
    return i;
}

int minhack_message_menu(int let, int how, const char *mesg)
{
    return -1;
}
